@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     :  Michael Hemmer <hemmer@mpi-inf.mpg.de> 
@@ -33,6 +34,7 @@
 #include <CGAL/Interval_nt.h>
 #include <CGAL/Arithmetic_kernel.h>
 #include <CGAL/convert_to_bfi.h>
+#include <CGAL/tss.h>
 
 #include <iterator>
 #include <list>
@@ -104,7 +106,7 @@ private:
   
 private: 
   static inline Self& get_default_instance(){
-    static Self x = Self(0); 
+    CGAL_STATIC_THREAD_LOCAL_VARIABLE(Self, x,0); 
     return x; 
   }
 public:
@@ -321,9 +323,10 @@ public:
   void refine_to(const Rational& lo, const Rational& hi) const {
     // test whether lo < x < hi
     // and refines isolating interval until in ]lo,hi[
-    CGAL::Comparison_result s;
-    s  = compare_distinct(lo); CGAL_assertion(CGAL::LARGER == s);
-    s = compare_distinct(hi) ; CGAL_assertion(CGAL::SMALLER == s);
+    CGAL_assertion_code(CGAL::Comparison_result s =) compare_distinct(lo);
+    CGAL_assertion(CGAL::LARGER == s);
+    CGAL_assertion_code(s =) compare_distinct(hi);
+    CGAL_assertion(CGAL::SMALLER == s);
   }
 
 
@@ -382,7 +385,7 @@ public:
   typedef internal::Algebraic_real_d_1< Coefficient, Rational, HandlePolicy, RepClass > Type;
 
   class Compare
-    : public std::binary_function< Type, Type, CGAL::Comparison_result > {
+    : public CGAL::binary_function< Type, Type, CGAL::Comparison_result > {
   public:
     CGAL::Comparison_result operator()( const Type& a, const Type& b ) const
     { return a.compare( b ); }
@@ -411,7 +414,7 @@ public:
   };
 
   class Sgn
-    : public std::unary_function< Type, CGAL::Sign > {
+    : public CGAL::unary_function< Type, CGAL::Sign > {
   public:
     CGAL::Sign operator()( const Type& a ) const {
       return a.compare( Rational(0) );
@@ -419,7 +422,7 @@ public:
   };
 
   class To_double
-    : public std::unary_function< Type, double > {
+    : public CGAL::unary_function< Type, double > {
   public:
     double operator()(const Type& a) const {
       return a.to_double();
@@ -427,7 +430,7 @@ public:
   };
 
   class To_interval
-    : public std::unary_function< Type, std::pair<double, double> > {
+    : public CGAL::unary_function< Type, std::pair<double, double> > {
   public:
     typename std::pair<double, double> operator()(const Type& a) const {
       return a.to_interval();
