@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Laurent RINEAU
@@ -33,7 +34,7 @@
 #include <utility>
 #include <iterator>
 #include <boost/iterator/filter_iterator.hpp>
-#include <boost/iterator/transform_iterator.hpp>
+#include <CGAL/boost/iterator/transform_iterator.hpp>
 
 namespace CGAL {
 
@@ -69,7 +70,7 @@ namespace Mesh_2 {
         typedef typename Tr::Edge Result_type;
 
         /** \param tr_ points to the triangulation. */
-        explicit Is_a_constrained_edge(const Tr& tr_) : tr(tr_) {}
+        explicit Is_a_constrained_edge(const Tr& tr_) : tr(tr_), i(-1) {}
 
         bool operator()(const Constrained_edge& ce)
         {
@@ -634,7 +635,7 @@ protected:
 
 private: /** \name DEBUGGING TYPES AND DATAS */
   class From_pair_of_vertex_to_edge 
-    : public std::unary_function<Constrained_edge, Edge>
+    : public CGAL::unary_function<Constrained_edge, Edge>
   {
     Tr& tr;
   public:
