@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Stéphane Tayeb
@@ -26,6 +27,10 @@
 #ifndef CGAL_MESH_3_LABELED_MESH_DOMAIN_3_H
 #define CGAL_MESH_3_LABELED_MESH_DOMAIN_3_H
 
+#include <CGAL/license/Mesh_3.h>
+
+#include <CGAL/disable_warnings.h>
+
 #define CGAL_DEPRECATED_HEADER "<CGAL/Mesh_3/Labeled_mesh_domain_3.h>"
 #define CGAL_REPLACEMENT_HEADER "<CGAL/Labeled_mesh_domain_3.h>"
 #include <CGAL/internal/deprecation_warning.h>
@@ -35,7 +40,6 @@
 #include <CGAL/Bbox_3.h>
 
 #include <CGAL/point_generators_3.h>
-#include <CGAL/Mesh_3/Creator_weighted_point_3.h>
 
 #include <boost/variant.hpp>
 #include <boost/format.hpp>
@@ -118,6 +122,13 @@ public:
       delete p_rng_;
   }
 
+  /**
+   * Returns a bounding box of the domain
+   */
+  Bbox_3 bbox() const {
+    return this->bbox_.bbox();
+  }
+
   /**
    * Constructs  a set of \ccc{n} points on the surface, and output them to
    *  the output iterator \ccc{pts} whose value type is required to be
@@ -620,4 +631,6 @@ Labeled_mesh_domain_3<F,BGT>::Construct_initial_points::operator()(
 
 }  // end namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // LABELLED_MESH_TRAITS_3_H_
