@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Stephane Tayeb
@@ -27,6 +28,7 @@
 
 #include <CGAL/license/Mesh_3.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/Mesh_3/config.h>
 
@@ -52,8 +54,8 @@ class Lloyd_move
   typedef typename Tr::Vertex_handle  Vertex_handle;
   typedef typename Tr::Edge           Edge;
   typedef typename Tr::Facet          Facet;
-  typedef typename Tr::Point          Point_3;  
-  typedef typename Point_3::Point     Bare_point_3;
+  typedef typename Tr::Weighted_point Weighted_point;
+  typedef typename Tr::Bare_point     Bare_point;
   
   typedef typename std::vector<Cell_handle>   Cell_vector;
   typedef typename std::vector<Vertex_handle> Vertex_vector;
@@ -112,27 +114,31 @@ public:
   
 private:
   /**
-   * Project_on_plane defines operator() to project Point_3 object on plane.
+   * Project_on_plane defines operator() to project point object on plane.
    */
   struct Project_on_plane
   {
-    Project_on_plane(const Plane_3& plane) : plane_(plane) {}
-    
-    Point_3 operator()(const Point_3& p) const
-    { return Gt().construct_projected_point_3_object()(plane_,p); }
+    Project_on_plane(const Plane_3& plane, const C3T3& c3t3)
+      : plane_(plane), c3t3_(c3t3)
+    { }
+
+    Bare_point operator()(const Bare_point& p) const
+    { return c3t3_.triangulation().geom_traits().
+          construct_projected_point_3_object()(plane_, p); }
     
   private:
     const Plane_3& plane_;
+    const C3T3& c3t3_;
   };
   
   /**
-   * To_2d defines operator() to transform Point_3 into Point_2
+   * To_2d defines operator() to transform Bare_point into Point_2
    */
   struct To_2d
   {
     To_2d(const Aff_transformation_3& to_2d) : to_2d_(to_2d) {}
     
-    Point_2 operator()(const Point_3& p) const
+    Point_2 operator()(const Bare_point& p) const
     { return Point_2(to_2d_.transform(p).x(), to_2d_.transform(p).y()); }
     
   private:
@@ -140,14 +146,14 @@ private:
   };
   
   /**
-   * To_3d defines operator() to transform Point_2 into Point_3
+   * To_3d defines operator() to transform Point_2 into Bare_point
    */
   struct To_3d
   {
     To_3d(const Aff_transformation_3& to_3d) : to_3d_(to_3d) {}
     
-    Point_3 operator()(const Point_2& p) const
-    { return to_3d_.transform(Point_3(Bare_point_3(p.x(),p.y(),0))); }
+    Bare_point operator()(const Point_2& p) const
+    { return to_3d_.transform((Bare_point(p.x(),p.y(),0))); }
     
   private:
     const Aff_transformation_3& to_3d_;
@@ -169,12 +175,10 @@ private:
     // -----------------------------------
     // Tessellate Voronoi cell
     // -----------------------------------
-    // get cells incident to v
-    const Tr& tr = c3t3.triangulation();
-    
+
     // Stores vertex that have already been used
     std::set<Vertex_handle> treated_vertex;
-    
+
     for (typename Cell_vector::const_iterator cit = incident_cells.begin();
          cit != incident_cells.end();
          ++cit)
@@ -190,7 +194,7 @@ private:
         
         // Vertex has not been treated: turn around edge(v,v1)
         treated_vertex.insert(v1);
-        turn_around_edge(v, Edge(*cit,k,(k+i)&3), tr,
+        turn_around_edge(v, Edge(*cit,k,(k+i)&3), c3t3,
                          move, sum_masses, sizing_field);
       }
     }
@@ -209,7 +213,7 @@ private:
     CGAL_precondition(c3t3.in_dimension(v) == 2);
     
     // get all surface delaunay ball point
-    std::vector<Point_3> points = extract_lloyd_boundary_points(v,c3t3);
+    std::vector<Bare_point> points = extract_lloyd_boundary_points(v,c3t3);
     
     switch(points.size())
     {
@@ -221,21 +225,21 @@ private:
       }
       case 2: // centroid 
       {
-        const Point_3& a = points.front();
-        const Point_3& b = points.back();
-        return centroid_segment_move(v,a,b,sizing_field);
+        const Bare_point& a = points.front();
+        const Bare_point& b = points.back();
+        return centroid_segment_move(v,a,b,c3t3,sizing_field);
         break;
       }
       case 3: // triangle centroid 
       {
-        const Point_3& a = points.at(0);
-        const Point_3& b = points.at(1);
-        const Point_3& c = points.at(2);
-        return centroid_triangle_move(v,a,b,c,sizing_field);
+        const Bare_point& a = points.at(0);
+        const Bare_point& b = points.at(1);
+        const Bare_point& c = points.at(2);
+        return centroid_triangle_move(v,a,b,c,c3t3,sizing_field);
         break;
       }
       default: // >= 4 points, centroid + projection
-        return centroid_general_move(v,points.begin(),points.end(),sizing_field);
+        return centroid_general_move(v,points.begin(),points.end(),c3t3,sizing_field);
         break;
     }
     
@@ -246,14 +250,14 @@ private:
    * Returns a vector containing surface delaunay ball center of surface
    * facets incident to vertex \c v
    */
-  std::vector<Point_3> extract_lloyd_boundary_points(const Vertex_handle& v,
-                                                     const C3T3& c3t3) const
+  std::vector<Bare_point> extract_lloyd_boundary_points(const Vertex_handle& v,
+                                                        const C3T3& c3t3) const
   {
     Facet_vector incident_facets;
     incident_facets.reserve(64);
     c3t3.triangulation().finite_incident_facets(v,std::back_inserter(incident_facets));
     
-    std::vector<Point_3> points;
+    std::vector<Bare_point> points;
     points.reserve(64);
     
     // Get c3t3's facets incident to v, and add their surface delaunay ball
@@ -275,15 +279,18 @@ private:
    * Return move from \c v to centroid of segment [a,b]
    */
   Vector_3 centroid_segment_move(const Vertex_handle& v,
-                                 const Point_3& a,
-                                 const Point_3& b,
+                                 const Bare_point& a,
+                                 const Bare_point& b,
+                                 const C3T3& c3t3,
                                  const Sizing_field& sizing_field) const
   {
     typename Gt::Construct_vector_3 vector =
-      Gt().construct_vector_3_object();
-    
-    const Point_3& p = v->point();
-    
+      c3t3.triangulation().geom_traits().construct_vector_3_object();
+    typename Gt::Construct_point_3 wp2p =
+      c3t3.triangulation().geom_traits().construct_point_3_object();
+
+    const Bare_point& p = wp2p(v->point());
+
     FT da = density_1d(a,v,sizing_field);
     FT db = density_1d(b,v,sizing_field);
 
@@ -295,15 +302,18 @@ private:
    * Return move from \c v to centroid of triangle [a,b,c]
    */
   Vector_3 centroid_triangle_move(const Vertex_handle& v,
-                                  const Point_3& a,
-                                  const Point_3& b,
-                                  const Point_3& c,
+                                  const Bare_point& a,
+                                  const Bare_point& b,
+                                  const Bare_point& c,
+                                  const C3T3& c3t3,
                                   const Sizing_field& sizing_field) const
   {
     typename Gt::Construct_vector_3 vector =
-      Gt().construct_vector_3_object();
-    
-    const Point_3& p = v->point();
+      c3t3.triangulation().geom_traits().construct_vector_3_object();
+    typename Gt::Construct_point_3 wp2p =
+      c3t3.triangulation().geom_traits().construct_point_3_object();
+
+    const Bare_point& p = wp2p(v->point());
     
     FT da = density_2d<true>(a,v,sizing_field);
     FT db = density_2d<false>(b,v,sizing_field);
@@ -322,21 +332,24 @@ private:
   Vector_3 centroid_general_move(const Vertex_handle& v,
                                  ForwardIterator first,
                                  ForwardIterator last,
+                                 const C3T3& c3t3,
                                  const Sizing_field& sizing_field) const
   {
     CGAL_assertion(std::distance(first,last) > 3);
     
     // Fit plane using point-based PCA: compute least square fitting plane
     Plane_3 plane;
-    Point_3 point;
-    CGAL::linear_least_squares_fitting_3(first,last,plane, point, Dimension_tag<0>(), Gt(),Default_diagonalize_traits<FT, 3>());
+    Bare_point point;
+    CGAL::linear_least_squares_fitting_3(first,last,plane, point, Dimension_tag<0>(),
+                                         c3t3.triangulation().geom_traits(),
+                                         Default_diagonalize_traits<FT, 3>());
     
     // Project all points to the plane
-    std::transform(first, last, first, Project_on_plane(plane));
+    std::transform(first, last, first, Project_on_plane(plane, c3t3));
     CGAL_assertion(std::distance(first,last) > 3);
     
     // Get 2D-3D transformations
-    Aff_transformation_3 to_3d = compute_to_3d_transform(plane, *first);
+    Aff_transformation_3 to_3d = compute_to_3d_transform(plane, *first, c3t3);
     Aff_transformation_3 to_2d = to_3d.inverse();
     
     // Transform to 2D
@@ -353,15 +366,14 @@ private:
                            std::back_inserter(ch_2d));
     
     // Lift back convex hull to 3D 
-    std::vector<Point_3> polygon_3d;
+    std::vector<Bare_point> polygon_3d;
     polygon_3d.reserve(ch_2d.size());
     std::transform(ch_2d.begin(), ch_2d.end(),
                    std::back_inserter(polygon_3d), To_3d(to_3d));
     
     // Compute centroid using quadrature sizing
-    return centroid_3d_polygon_move(v,
-                                    polygon_3d.begin(), polygon_3d.end(),
-                                    sizing_field);
+    return centroid_3d_polygon_move(v, polygon_3d.begin(), polygon_3d.end(),
+                                    c3t3, sizing_field);
   }
   
   /**
@@ -372,30 +384,31 @@ private:
   Vector_3 centroid_3d_polygon_move(const Vertex_handle& v,
                                     ForwardIterator first,
                                     ForwardIterator last,
+                                    const C3T3& c3t3,
                                     const Sizing_field& sizing_field) const
   {
     CGAL_precondition(std::distance(first,last) >= 3);
     
     typename Gt::Construct_vector_3 vector =
-      Gt().construct_vector_3_object();
-    
+      c3t3.triangulation().geom_traits().construct_vector_3_object();
     typename Gt::Construct_centroid_3 centroid =
-      Gt().construct_centroid_3_object();
-    
+      c3t3.triangulation().geom_traits().construct_centroid_3_object();
     typename Gt::Compute_area_3 area = 
-      Gt().compute_area_3_object();
-    
+      c3t3.triangulation().geom_traits().compute_area_3_object();
+    typename Gt::Construct_point_3 cp =
+      c3t3.triangulation().geom_traits().construct_point_3_object();
+
     // Vertex current position
-    const Point_3& vertex_position = v->point();
+    const Bare_point& vertex_position = cp(v->point());
     
     // Use as reference point to triangulate
-    const Point_3& a = *first++;
-    const Point_3* b = &(*first++);
+    const Bare_point& a = *first++;
+    const Bare_point* b = &(*first++);
     
     // Treat first point (optimize density_2d call)
-    const Point_3& c = *first++;
+    const Bare_point& c = *first++;
     
-    Point_3 triangle_centroid = centroid(a,*b,c);
+    const Bare_point& triangle_centroid = centroid(a,*b,c);
     FT density = density_2d<true>(triangle_centroid, v, sizing_field);
     
     FT sum_masses = density * area(a,*b,c);
@@ -406,9 +419,9 @@ private:
     // Next points
     while ( first != last )
     {
-      const Point_3& c = *first++;
+      const Bare_point& c = *first++;
 
-      Point_3 triangle_centroid = centroid(a,*b,c);
+      const Bare_point& triangle_centroid = centroid(a,*b,c);
       FT density = density_2d<false>(triangle_centroid, v, sizing_field);
       FT mass = density * area(a,*b,c);
       
@@ -427,14 +440,14 @@ private:
    * Returns the transformation from reference_point to plane
    */
   Aff_transformation_3 compute_to_3d_transform(const Plane_3& plane,
-                                               const Point_3& reference_point) const
+                                               const Bare_point& reference_point,
+                                               const C3T3& c3t3) const
   {
     typename Gt::Construct_base_vector_3 base =
-      Gt().construct_base_vector_3_object();
-    
+      c3t3.triangulation().geom_traits().construct_base_vector_3_object();
     typename Gt::Construct_orthogonal_vector_3 orthogonal_vector =
-      Gt().construct_orthogonal_vector_3_object();
-    
+      c3t3.triangulation().geom_traits().construct_orthogonal_vector_3_object();
+
     Vector_3 u = base(plane, 1);
     u = u / CGAL::sqrt(u*u);
     
@@ -453,7 +466,7 @@ private:
    * returns density_1d
    */
   template <typename Sizing_field>
-  FT density_1d(const Point_3& p,
+  FT density_1d(const Bare_point& p,
                 const Vertex_handle& v,
                 const Sizing_field& sizing_field) const
   {
@@ -468,7 +481,7 @@ private:
    * returns density_2d
    */
   template <bool use_v, typename Sizing_field>
-  FT density_2d(const Point_3& p,
+  FT density_2d(const Bare_point& p,
                 const Vertex_handle& v,
                 const Sizing_field& sizing_field) const
   {
@@ -483,7 +496,7 @@ private:
    * returns density_3d
    */
   template <typename Sizing_field>
-  FT density_3d(const Point_3& p,
+  FT density_3d(const Bare_point& p,
                 const Cell_handle& cell,
                 const Sizing_field& sizing_field) const
   {
@@ -503,7 +516,7 @@ private:
    */
   void turn_around_edge(const Vertex_handle& v,
                         const Edge& edge,
-                        const Tr& tr,
+                        const C3T3& c3t3,
                         Vector_3& move,
                         FT& sum_masses,
                         const Sizing_field& sizing_field) const
@@ -511,31 +524,30 @@ private:
     typedef typename Tr::Cell_circulator Cell_circulator;
     
     typename Gt::Compute_volume_3 volume = 
-      Gt().compute_volume_3_object();
-    
+      c3t3.triangulation().geom_traits().compute_volume_3_object();
     typename Gt::Construct_centroid_3 centroid =
-      Gt().construct_centroid_3_object();
-    
+      c3t3.triangulation().geom_traits().construct_centroid_3_object();
     typename Gt::Construct_vector_3 vector =
-      Gt().construct_vector_3_object();
+      c3t3.triangulation().geom_traits().construct_vector_3_object();
+    typename Gt::Construct_point_3 wp2p =
+      c3t3.triangulation().geom_traits().construct_point_3_object();
     
-    Cell_circulator current_cell = tr.incident_cells(edge);
+    Cell_circulator current_cell = c3t3.triangulation().incident_cells(edge);
     Cell_circulator done = current_cell;
     
     // a & b are fixed points
-    const Point_3& a = v->point();
-    const Point_3 b = tr.dual(current_cell++);
+    const Bare_point& a = wp2p(v->point());
+    const Bare_point& b = c3t3.triangulation().dual(current_cell++);
     CGAL_assertion(current_cell != done);
     
     // c & d are moving points
-    Point_3 c = tr.dual(current_cell++);
+    Bare_point c = c3t3.triangulation().dual(current_cell++);
     CGAL_assertion(current_cell != done);
     
     while ( current_cell != done )
     {
-      const Point_3 d = tr.dual(current_cell++);
-      
-      Point_3 tet_centroid = centroid(a,b,c,d);
+      const Bare_point& d = c3t3.triangulation().dual(current_cell++);
+      const Bare_point& tet_centroid = centroid(a,b,c,d);
       
       // Compute mass
       FT density = density_3d(tet_centroid, current_cell, sizing_field);
@@ -555,4 +567,6 @@ private:
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_MESH_3_LLOYD_MOVE_H
