@@ -14,11 +14,15 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Marc Pouget and Frédéric Cazals
 #ifndef CGAL_MONGE_VIA_JET_FITTING_H_
 #define CGAL_MONGE_VIA_JET_FITTING_H_
 
+#include <CGAL/license/Jet_fitting_3.h>
+
+
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/circulator.h>
 #include <CGAL/Linear_algebraCd.h>
@@ -30,7 +34,7 @@
 #include <CGAL/Eigen_svd.h>
 #else
 #ifdef CGAL_LAPACK_ENABLED
-#include <CGAL/Lapack/Linear_algebra_lapack.h>
+#include <CGAL/Lapack_svd.h>
 #endif
 #endif
 
