@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Ilker O. Yaz
@@ -21,19 +22,22 @@
 #ifndef CGAL_HOLE_FILLING_TRIANGULATE_HOLE_POLYLINE_H
 #define CGAL_HOLE_FILLING_TRIANGULATE_HOLE_POLYLINE_H
 
-#include <CGAL/Mesh_3/dihedral_angle_3.h>
+#include <CGAL/license/Polygon_mesh_processing/meshing_hole_filling.h>
+
+
 #include <CGAL/value_type_traits.h>
 #include <CGAL/Delaunay_triangulation_3.h>
 #include <CGAL/Triangulation_vertex_base_with_info_3.h>
 #include <CGAL/iterator.h>
-#include <CGAL/trace.h>
 #include <CGAL/use.h>
+#include <CGAL/Kernel/global_functions_3.h>
+#include <CGAL/squared_distance_3.h>
 
 #include <vector>
 #include <stack>
 #include <map>
 
-#include <boost/iterator/transform_iterator.hpp>
+#include <CGAL/boost/iterator/transform_iterator.hpp>
 #include <boost/unordered_set.hpp>
 
 namespace CGAL {
@@ -247,14 +251,14 @@ private:
       // check whether the edge is border
       if( (v0 + 1 == v1 || (v0 == n-1 && v1 == 0) ) && !Q.empty() ) {
         angle = 180 - CGAL::abs( 
-           to_double(CGAL::Mesh_3::dihedral_angle(P[v0],P[v1],P[v_other],Q[v0])) );
+           to_double(CGAL::approximate_dihedral_angle(P[v0],P[v1],P[v_other],Q[v0])) );
       }
       else {
         if(e == 2) { continue; }
         if(lambda.get(v0, v1) != -1){
           const Point_3& p01 = P[lambda.get(v0, v1)];
           angle = 180 - CGAL::abs( 
-            to_double(CGAL::Mesh_3::dihedral_angle(P[v0],P[v1],P[v_other],p01)) );
+            to_double(CGAL::approximate_dihedral_angle(P[v0],P[v1],P[v_other],p01)) );
         }
       }
       ang_max = (std::max)(ang_max, angle);
@@ -476,7 +480,7 @@ struct Tracer_polyline_incomplete {
       }
 
       CGAL_assertion(la >= 0 && la < n);
-      CGAL_assertion(r.first < la && r.second > la);
+      CGAL_assertion( (r.first < la) && (r.second > la) );
       *out++ = OutputIteratorValueType(r.first, la, r.second);
 
       ranges.push(std::make_pair(r.first, la));
@@ -1138,7 +1142,7 @@ public:
     
     triangulate_all(P, Q, WC, std::make_pair(0,n-1), W, lambda);
 
-    if(W.get(0,n-1) == Weight::NOT_VALID()) {
+    if(W.get(0,n-1) == Weight::NOT_VALID() || n <= 2) {
       #ifndef CGAL_TEST_SUITE
       CGAL_warning(!"Returning no output. No possible triangulation is found!");
       #else
@@ -1225,7 +1229,9 @@ triangulate_hole_polyline(const PointRange1& points,
   typename WeightCalculator::Weight w = use_delaunay_triangulation ?
     Fill_DT().operator()(P,Q,tracer,WC) :
     Fill().operator()(P,Q,tracer,WC);
-  CGAL_TRACE_STREAM << w << std::endl;
+  #ifdef CGAL_PMP_HOLE_FILLING_DEBUG
+  std::cerr << w << std::endl;
+  #endif
   return w;
 }
 
