@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Laurent Rineau and Sebastien Loriot
@@ -24,6 +25,7 @@
 
 #include <CGAL/license/Polygon_mesh_processing/orientation.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/tuple.h>
 #include <CGAL/array.h>
@@ -489,4 +491,6 @@ bool orient_polygon_soup(PointRange& points,
 
 } }//end namespace CGAL::Polygon_mesh_processing
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_POLYGON_MESH_PROCESSING_ORIENT_POLYGON_SOUP
