@@ -15,7 +15,8 @@
 //
 // $URL$
 // $Id$
-// 
+// SPDX-License-Identifier: GPL-3.0+
+//
 //
 // Author(s)     : Pierre Alliez, Laurent Rineau, Ilker O. Yaz
 
@@ -25,6 +26,10 @@
 #ifndef CGAL_POLYGON_MESH_PROCESSING_SELF_INTERSECTIONS
 #define CGAL_POLYGON_MESH_PROCESSING_SELF_INTERSECTIONS
 
+#include <CGAL/license/Polygon_mesh_processing/predicate.h>
+
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/box_intersection_d.h>
 #include <CGAL/intersections.h>
 #include <CGAL/Bbox_3.h>
@@ -45,6 +50,11 @@
 #include <CGAL/Polygon_mesh_processing/internal/named_function_params.h>
 #include <CGAL/Polygon_mesh_processing/internal/named_params_helper.h>
 
+#ifdef DOXYGEN_RUNNING
+#define CGAL_PMP_NP_TEMPLATE_PARAMETERS NamedParameters
+#define CGAL_PMP_NP_CLASS NamedParameters
+#endif
+
 namespace CGAL {
 namespace internal {
 template <class TM,//TriangleMesh
@@ -55,7 +65,7 @@ template <class TM,//TriangleMesh
 struct Intersect_facets
 {
   // wrapper to check whether anything is inserted to output iterator
-  struct Output_iterator_with_bool 
+  struct Output_iterator_with_bool
   {
     Output_iterator_with_bool(OutputIterator* out, bool* intersected)
       : m_iterator(out), m_intersected(intersected) { }
@@ -63,7 +73,7 @@ struct Intersect_facets
     template<class T>
     void operator()(const T& t) {
       *m_intersected = true;
-      *(*m_iterator)++ = t; 
+      *(*m_iterator)++ = t;
     }
 
     OutputIterator* m_iterator;
@@ -86,9 +96,9 @@ struct Intersect_facets
   typename Kernel::Construct_triangle_3 triangle_functor;
   typename Kernel::Do_intersect_3       do_intersect_3_functor;
 
-  
+
   Intersect_facets(const TM& tmesh, OutputIterator it, VertexPointMap vpmap, const Kernel& kernel)
-    : 
+    :
     m_tmesh(tmesh),
     m_vpmap(vpmap),
     m_iterator(it),
@@ -159,7 +169,7 @@ struct Intersect_facets
     }
 
     if(v != halfedge_descriptor()){
-      // found shared vertex: 
+      // found shared vertex:
       CGAL_assertion(target(h,m_tmesh) == target(v,m_tmesh));
       // geometric check if the opposite segments intersect the triangles
       Triangle t1 = triangle_functor( get(m_vpmap,target(h,m_tmesh)),
@@ -168,12 +178,12 @@ struct Intersect_facets
       Triangle t2 = triangle_functor( get(m_vpmap, target(v,m_tmesh)),
                                       get(m_vpmap, target(next(v,m_tmesh),m_tmesh)),
                                       get(m_vpmap, target(next(next(v,m_tmesh),m_tmesh),m_tmesh)));
-      
+
       Segment s1 = segment_functor( get(m_vpmap, target(next(h,m_tmesh),m_tmesh)),
                                     get(m_vpmap, target(next(next(h,m_tmesh),m_tmesh),m_tmesh)));
       Segment s2 = segment_functor( get(m_vpmap, target(next(v,m_tmesh),m_tmesh)),
                                     get(m_vpmap, target(next(next(v,m_tmesh),m_tmesh),m_tmesh)));
-      
+
       if(do_intersect_3_functor(t1,s2)){
         *m_iterator_wrapper++ = std::make_pair(b->info(), c->info());
       } else if(do_intersect_3_functor(t2,s1)){
@@ -181,7 +191,7 @@ struct Intersect_facets
       }
       return;
     }
-    
+
     // check for geometric intersection
     Triangle t1 = triangle_functor( get(m_vpmap, target(h,m_tmesh)),
                                     get(m_vpmap, target(next(h,m_tmesh),m_tmesh)),
@@ -222,25 +232,29 @@ self_intersections( const FaceRange& face_range,
                     const NamedParameters& np);
 #endif
 
-/** 
+/**
  * \ingroup PMP_intersection_grp
  * detects and records self-intersections of a triangulated surface mesh.
  * This function depends on the package \ref PkgBoxIntersectionDSummary
  * @pre `CGAL::is_triangle_mesh(tmesh)`
  *
- * @tparam TriangleMesh a model of `FaceListGraph` that has an internal property map
-*         for `CGAL::vertex_point_t`
- * @tparam OutputIterator a model of `OutputIterator` holding objects of type 
+ * @tparam TriangleMesh a model of `FaceListGraph`
+ * @tparam OutputIterator a model of `OutputIterator` holding objects of type
  *   `std::pair<boost::graph_traits<TriangleMesh>::%face_descriptor, boost::graph_traits<TriangleMesh>::%face_descriptor>`
- * @tparam NamedParameters a sequence of \ref namedparameters
+ * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
  *
  * @param tmesh the triangulated surface mesh to be checked
- * @param out output iterator to be filled with all pairs of non-adjacent faces that intersect
- * @param np optional sequence of \ref namedparameters among the ones listed below
+ * @param out output iterator to be filled with all pairs of non-adjacent faces that intersect.
+              In case `tmesh` contains some degenerate faces, for each degenerate face `f` a pair `(f,f)`
+              will be put in `out` before any other self intersection between non-degenerate faces.
+              These are the only pairs where degenerate faces will be reported.
+ * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `SelfIntersectionTraits` \cgalParamEnd
+ *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+ *   If this parameter is omitted, an internal property map for
+ *   `CGAL::vertex_point_t` should be available in `TriangleMesh`\cgalParamEnd
+ *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
  * \cgalNamedParamsEnd
  *
  * @return `out`
@@ -259,7 +273,7 @@ self_intersections(const TriangleMesh& tmesh
 #ifdef DOXYGEN_RUNNING
                  , const NamedParameters& np)
 #else
-                 , const pmp_bgl_named_params<P,T,R>& np)
+                 , const cgal_bgl_named_params<P,T,R>& np)
 #endif
 {
   return self_intersections(faces(tmesh), tmesh, out, np);
@@ -282,23 +296,24 @@ self_intersections(const TriangleMesh& tmesh, OutputIterator out)
  *
  * @pre `CGAL::is_triangle_mesh(tmesh)`
  *
- * @tparam FaceRange range of `boost::graph_traits<PolygonMesh>::%face_descriptor`,
+ * @tparam FaceRange range of `boost::graph_traits<TriangleMesh>::%face_descriptor`,
  *  model of `Range`.
  * Its iterator type is `RandomAccessIterator`.
- * @tparam TriangleMesh a model of `FaceListGraph` that has an internal property map
- *         for `CGAL::vertex_point_t`
+ * @tparam TriangleMesh a model of `FaceListGraph`
  * @tparam OutputIterator a model of `OutputIterator` holding objects of type
  *   `std::pair<boost::graph_traits<TriangleMesh>::%face_descriptor, boost::graph_traits<TriangleMesh>::%face_descriptor>`
- * @tparam NamedParameters a sequence of \ref namedparameters
+ * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
  *
  * @param face_range the range of faces to check for self-intersection.
  * @param tmesh the triangulated surface mesh to be checked
  * @param out output iterator to be filled with all pairs of non-adjacent faces that intersect
- * @param np optional sequence of \ref namedparameters among the ones listed below
+ * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh` \cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `SelfIntersectionTraits` \cgalParamEnd
+ *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `pmesh`.
+ *   If this parameter is omitted, an internal property map for
+ *   `CGAL::vertex_point_t` should be available in `TriangleMesh`\cgalParamEnd
+ *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
  * \cgalNamedParamsEnd
 
  */
@@ -326,16 +341,20 @@ self_intersections( const FaceRange& face_range,
   );
 
   typedef typename GetVertexPointMap<TM, NamedParameters>::const_type VertexPointMap;
-  VertexPointMap vpmap = choose_const_pmap(get_param(np, boost::vertex_point),
-                                           tmesh,
-                                           boost::vertex_point);
+  VertexPointMap vpmap = boost::choose_param(get_param(np, internal_np::vertex_point),
+                                             get_const_property_map(boost::vertex_point, tmesh));
 
   BOOST_FOREACH(face_descriptor f, face_range)
   {
-    boxes.push_back(Box( get(vpmap, target(halfedge(f,tmesh),tmesh)).bbox()
-      + get(vpmap, target(next(halfedge(f, tmesh), tmesh), tmesh)).bbox()
-      + get(vpmap, target(next(next(halfedge(f, tmesh), tmesh), tmesh), tmesh)).bbox(),
-    f));
+    typename boost::property_traits<VertexPointMap>::reference
+      p = get(vpmap, target(halfedge(f,tmesh),tmesh)),
+      q = get(vpmap, target(next(halfedge(f, tmesh), tmesh), tmesh)),
+      r = get(vpmap, target(next(next(halfedge(f, tmesh), tmesh), tmesh), tmesh));
+
+    if ( collinear(p, q, r) )
+      *out++= std::make_pair(f,f);
+    else
+      boxes.push_back(Box(p.bbox() + q.bbox() + r.bbox(), f));
   }
   // generate box pointers
   std::vector<const Box*> box_ptr;
@@ -348,7 +367,7 @@ self_intersections( const FaceRange& face_range,
   typedef typename GetGeomTraits<TM, NamedParameters>::type GeomTraits;
   CGAL::internal::Intersect_facets<TM,GeomTraits,Box,OutputIterator,VertexPointMap>
     intersect_facets(tmesh, out, vpmap,
-      boost::choose_param(get_param(np, geom_traits), GeomTraits()));
+      boost::choose_param(get_param(np, internal_np::geom_traits), GeomTraits()));
 
   std::ptrdiff_t cutoff = 2000;
   CGAL::box_self_intersection_d(box_ptr.begin(), box_ptr.end(),intersect_facets,cutoff);
@@ -375,25 +394,26 @@ OutputIterator self_intersections(const FaceRange& face_range,
  * This function depends on the package \ref PkgBoxIntersectionDSummary
  * @pre `CGAL::is_triangle_mesh(tmesh)`
  *
- * @tparam TriangleMesh a model of `FaceListGraph` that has an internal property map
- *         for `CGAL::vertex_point_t`
- * @tparam NamedParameters a sequence of \ref namedparameters
+ * @tparam TriangleMesh a model of `FaceListGraph`
+ * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
  *
  * @param tmesh the triangulated surface mesh to be tested
- * @param np optional sequence of \ref namedparameters among the ones listed below
+ * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
  *
  * \cgalNamedParamsBegin
- *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tmesh` \cgalParamEnd
- *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `SelfIntersectionTraits` \cgalParamEnd
+ *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tmesh`.
+ *   If this parameter is omitted, an internal property map for
+ *   `CGAL::vertex_point_t` should be available in `TriangleMesh`\cgalParamEnd
+ *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `PMPSelfIntersectionTraits` \cgalParamEnd
  * \cgalNamedParamsEnd
  *
  * @return true if `tmesh` self-intersects
  */
 template <class TriangleMesh
-        , class NamedParameters
+        , class CGAL_PMP_NP_TEMPLATE_PARAMETERS
           >
 bool does_self_intersect(const TriangleMesh& tmesh
-                        , const NamedParameters& np)
+                        , const CGAL_PMP_NP_CLASS& np)
 {
   CGAL_precondition(CGAL::is_triangle_mesh(tmesh));
 
@@ -402,7 +422,51 @@ bool does_self_intersect(const TriangleMesh& tmesh
     typedef boost::function_output_iterator<CGAL::internal::Throw_at_output> OutputIterator;
     self_intersections(tmesh, OutputIterator(), np);
   }
-  catch( CGAL::internal::Throw_at_output::Throw_at_output_exception& ) 
+  catch( CGAL::internal::Throw_at_output::Throw_at_output_exception& )
+  { return true; }
+
+  return false;
+}
+
+/**
+ * \ingroup PMP_intersection_grp
+ * tests if a set of faces of a triangulated surface mesh self-intersects.
+ * This function depends on the package \ref PkgBoxIntersectionDSummary
+ * @pre `CGAL::is_triangle_mesh(tmesh)`
+ *
+ * @tparam FaceRange a range of `face_descriptor`
+ * @tparam TriangleMesh a model of `FaceListGraph`
+ * @tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
+ *
+ * @param face_range the set of faces to test for self-intersection
+ * @param tmesh the triangulated surface mesh to be tested
+ * @param np optional sequence of \ref pmp_namedparameters "Named Parameters" among the ones listed below
+ *
+ * \cgalNamedParamsBegin
+ *    \cgalParamBegin{vertex_point_map} the property map with the points associated to the vertices of `tmesh`.
+ *   If this parameter is omitted, an internal property map for
+ *   `CGAL::vertex_point_t` should be available in `TriangleMesh`\cgalParamEnd
+ *    \cgalParamBegin{geom_traits} an instance of a geometric traits class, model of `SelfIntersectionTraits` \cgalParamEnd
+ * \cgalNamedParamsEnd
+ *
+ * @return true if the faces in `face_range` self-intersects
+ */
+template <class FaceRange,
+          class TriangleMesh,
+          class NamedParameters
+          >
+bool does_self_intersect(const FaceRange& face_range,
+                         const TriangleMesh& tmesh,
+                         const NamedParameters& np)
+{
+  CGAL_precondition(CGAL::is_triangle_mesh(tmesh));
+
+  try
+  {
+    typedef boost::function_output_iterator<CGAL::internal::Throw_at_output> OutputIterator;
+    self_intersections(face_range, tmesh, OutputIterator(), np);
+  }
+  catch( CGAL::internal::Throw_at_output::Throw_at_output_exception& )
   { return true; }
 
   return false;
@@ -415,10 +479,21 @@ bool does_self_intersect(const TriangleMesh& tmesh)
   return does_self_intersect(tmesh,
     CGAL::Polygon_mesh_processing::parameters::all_default());
 }
+
+template <class FaceRange, class TriangleMesh>
+bool does_self_intersect(const FaceRange& face_range,
+                         const TriangleMesh& tmesh)
+{
+  return does_self_intersect(face_range, tmesh,
+    CGAL::Polygon_mesh_processing::parameters::all_default());
+}
+
 /// \endcond
 
 }// end namespace Polygon_mesh_processing
 
 }// namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_POLYGON_MESH_PROCESSING_SELF_INTERSECTIONS
