@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Bernd Gaertner <gaertner@inf.ethz.ch>
@@ -23,6 +24,8 @@
 
 #include <CGAL/license/QP_solver.h>
 
+#include <CGAL/disable_warnings.h>
+
 // this file defines a class for passing options to the linear and
 // quadratic programming solvers
 
@@ -139,4 +142,6 @@ std::ostream& operator<< (std::ostream& o,
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_QP_OPTIONS_H
